@@ -516,10 +516,26 @@ class PadRefCountTest(TestCase):
             self.assertEquals(self.gccollect(), 1) # collected the pad
         gst.debug('going into teardown')
 
-class PadBlockRefcountTest(TestCase):
+class PadBlockTest(TestCase):
+    def testCallbackFlush(self):
+        # check that the same block callback can be called more than once (weird
+        # test but it was broken)
+
+        def blocked_cb(pad, blocked):
+            pad.push_event(gst.event_new_flush_start())
+       
+        pad = gst.Pad('src', gst.PAD_SRC)
+        pad.set_active(True)
+        pad.set_blocked_async(True, blocked_cb)
+
+        for i in xrange(10):
+            buf = gst.Buffer('ciao')
+            pad.push(buf)
+            pad.push_event(gst.event_new_flush_stop())
+
     def testCallbackRefcount(self):
         def blocked_cb(pad, blocked):
-            self.assertTrue(pad.set_blocked_async(False, unblocked_cb))
+            pad.set_blocked_async(False, unblocked_cb)
 
         def unblocked_cb(pad, blocked):
             pass
@@ -527,27 +543,20 @@ class PadBlockRefcountTest(TestCase):
         cb_refcount = sys.getrefcount(blocked_cb)
         # sys.getrefcount() returns refcount + 1
         self.assertEquals(cb_refcount, 2)
-
-        fakesrc = gst.element_factory_make('fakesrc')
-        fakesrc.props.num_buffers = 2
-        fakesink = gst.element_factory_make('fakesink')
-
-        pipeline = gst.Pipeline()
-        pipeline.add(fakesrc, fakesink)
-
-        fakesrc.link(fakesink)
-
-        pad = fakesrc.get_pad('src')
+       
+        pad = gst.Pad('src', gst.PAD_SRC)
+        pad.set_active(True)
         pad.set_blocked_async(True, blocked_cb)
-
-        pipeline.set_state(gst.STATE_PLAYING)
-        pipeline.get_bus().poll(gst.MESSAGE_EOS, -1)
-        pipeline.set_state(gst.STATE_NULL)
-
-        # check that we don't leak a ref to the callback
+        # set_blocked_async refs the callback
+        self.assertEquals(sys.getrefcount(blocked_cb), 3)
+            
+        buf = gst.Buffer('ciao')
+        pad.push(buf)
+        
+        # in blocked_cb() we called set_blocked_async() with a different
+        # callback, so blocked_cb() should have been unreffed
         cb_refcount_after = sys.getrefcount(blocked_cb)
-        self.assertEquals(cb_refcount_after, cb_refcount)
-
+        self.assertEquals(sys.getrefcount(blocked_cb), cb_refcount)
 
 if __name__ == "__main__":
     unittest.main()
